//
// Showing off surface reding functions.
//
//! \file main.c
//! \author J Vijn
//! \date 20080514 - 20080514
//
/* === NOTES ===
*/

#include <stdlib.h>
#include <string.h>
#include <tonc.h>

#include "menu.h"

#include "all_gfx.h"

// --------------------------------------------------------------------
// TYPES 
// --------------------------------------------------------------------

// --------------------------------------------------------------------
// GLOBALS 
// --------------------------------------------------------------------

// --------------------------------------------------------------------
// FUNCTIONS 
// --------------------------------------------------------------------


//! Internal routine for testing surface procedures.
void test_surface_procs(const TSurface *src, TSurface *dst, 
	const TSurfaceProcTab *procs, u16 colors[])
{
	// Init object text
	tte_init_obj(&oam_mem[127], ATTR0_TALL, ATTR1_SIZE_8, 512, 
		CLR_YELLOW, 0, &vwf_default, NULL);
	tte_init_con();
	tte_set_margins(8, 140, 160, 152);

	// And go!
	tte_printf("#{es;P}%s surface primitives#{w:60}", procs->name);

	tte_printf("#{es;P}Rect#{w:20}");
	procs->rect(dst, 20, 20, 100, 100, colors[0]);

	tte_printf("#{w:30;es;P}Frame#{w:20}");
	procs->frame(dst, 21, 21, 99, 99, colors[1]);

	tte_printf("#{w:30;es;P}Hlines#{w:20}");

	procs->hline(dst, 23, 23, 96, colors[2]);
	procs->hline(dst, 23, 96, 96, colors[2]);

	tte_printf("#{w:30;es;P}Vlines#{w:20}");
	procs->vline(dst, 23, 25, 94, colors[3]);
	procs->vline(dst, 96, 25, 94, colors[3]);

	tte_printf("#{w:30;es;P}Lines#{w:20}");
	procs->line(dst, 25, 25, 94, 40, colors[4]);
	procs->line(dst, 94, 25, 79, 94, colors[4]);
	procs->line(dst, 94, 94, 25, 79, colors[4]);
	procs->line(dst, 25, 94, 40, 25, colors[4]);

	tte_printf("#{w:30;es;P}Full blit#{w:20}");
	procs->blit(dst, 120, 16, src->width, src->height, src, 0, 0);

	tte_printf("#{w:30;es;P}Partial blit#{w:20}");
	procs->blit(dst, 40, 40, 40, 40, src, 12, 8);

	tte_printf("#{w:30;es;P}Floodfill#{w:20}");
	procs->flood(dst, 40, 32, colors[5]);
	tte_printf("#{w:30;es;P}Again !#{w:20}");
	procs->flood(dst, 40, 32, colors[6]);

	tte_printf("#{w:30;es;P;w:30}Ta-dah!!!#{w:20}");

	key_wait_till_hit(KEY_ANY);
}

void test_bmp16_procs()
{
	TSurface turret;

	// Init turret for blitting.
	srf_init(&turret, SRF_BMP16, turretBmp16Bitmap, 128, 128, 4, NULL);

	REG_DISPCNT= DCNT_MODE3 | DCNT_BG2 | DCNT_OBJ | DCNT_OBJ_1D;

	u16 colors[8]= { CLR_GRAY, CLR_BLUE, CLR_RED, 
		CLR_LIME, CLR_MAG, turretBmp8Pal[0], CLR_YELLOW, 0 };
	test_surface_procs(&turret, &m3_surface, &bmp16_tab, colors);
}

void test_bmp8_procs()
{
	TSurface turret;

	// Init turret for blitting.
	srf_init(&turret, SRF_BMP8, turretBmp8Bitmap, 128, 128, 4, NULL);

	GRIT_CPY(pal_bg_mem, turretBmp8Pal);

	REG_DISPCNT= DCNT_MODE4 | DCNT_BG2 | DCNT_OBJ | DCNT_OBJ_1D;

	u16 colors[8]= { 6, 13, 1, 14, 15, 0, 14, 0 };
	test_surface_procs(&turret, &m4_surface, &bmp8_tab, colors);
}

void test_chr4c_procs()
{
	TSurface turret, dst;

	// Init turret for blitting.
	srf_init(&turret, SRF_CHR4C, turretChr4cTiles, 128, 128, 4, NULL);

	// Init destination surface
	srf_init(&dst, SRF_CHR4C, tile_mem[0], 240, 160, 4, pal_bg_mem);
	schr4c_prep_map(&dst, se_mem[31], 0);
	GRIT_CPY(pal_bg_mem, turretChr4cPal);

	REG_DISPCNT= DCNT_MODE0 | DCNT_BG2 | DCNT_OBJ | DCNT_OBJ_1D;
	REG_BG2CNT= BG_CBB(0)|BG_SBB(31);

	u16 colors[8]= { 6, 13, 1, 14, 15, 0, 14, 0 };
	test_surface_procs(&turret, &dst, &chr4c_tab, colors);
}



const TMenuItem cUnits[]=
{
	{
		"bmp16 procs", 
		"Demo of 16bpp bitmap surface procedures.",
		test_bmp16_procs
	}, 
	{
		"bmp8 procs", 
		"Demo of 8bpp bitmap surface procedures.",
		test_bmp8_procs
	}, 
	{
		"chr4 procs", 
		"Demo of 4bpp tiled surface procedures.",
		test_chr4c_procs
	}, 
};


int main()
{
	irq_init(NULL);
	irq_add(II_VBLANK, NULL);

	//test_chr4c_procs();


	TMenu menu;
	menu_create(&menu, cUnits, countof(cUnits));
	
	while(1)
	{
		menu_init(&menu);
		menu_run(&menu);
		menu_act(&menu);
	}

	while(1)
		VBlankIntrWait();

	return 0;
}

// EOF
